﻿using System;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Xml.Linq;
using CloudShot.Core.Interfaces;
using CloudShot.Core.Types;
using CloudShot.Core.Utils;
using CloudShot.Properties;

namespace CloudShot.ImageStorages
{
	public class FileSystemStorage : IImageStorage
	{
		public Image Logo { get { return Resources.folder; } }
		public string Name { get { return "Local Folder"; } }
		public string Description
		{
			get { return "Image will be saved to local folder and path to image will be copied to clipboard"; }
		}
		public AuthorizationMode AuthorizationMode { get { return AuthorizationMode.None; } }
		public string OAuthTokens { get; set; }
		public bool IsAuthorized { get { return true; } }

		public ImageSaveFormat[] SupportedFormats
		{
			get { return new[] { ImageSaveFormat.Png, ImageSaveFormat.Jpg, ImageSaveFormat.Jpeg }; }
		}

		public StorageType SupportedStorageType { get { return StorageType.CustomLocal; } }

		private ImageSaveFormat _currentImageFormat;
		public ImageSaveFormat CurrentImageFormat
		{
			get { return _currentImageFormat; }
			set
			{
				if (!SupportedFormats.Contains(value))
					_currentImageFormat = SupportedFormats.First();
				else
					_currentImageFormat = value;
			}
		}

		private string _folderToSave = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyPictures), "CloudShot");
		public string FolderToSave
		{
			get { return _folderToSave; }
			set { _folderToSave = value; }
		}

		public void SignInAsync(Action<bool> callback)
		{
		}

		public void SignOut()
		{
		}

		private ShotInfo SaveImage(Image image, string shotName)
		{
			Directory.CreateDirectory(FolderToSave);//create directory if not exist
			string pathToSave = Path.Combine(FolderToSave, shotName);

			using (MemoryStream ms = new MemoryStream())
			using (FileStream fs = new FileStream(pathToSave, FileMode.Create))
			{
				image.Save(ms, CurrentImageFormat.GetImageFormat());
				ms.WriteTo(fs);
			}

			return new ShotInfo(shotName, pathToSave, image);
		}

		public ShotInfo SaveImageAsync(Image image, string name, Action<ShotInfo> uploadCompleted)
		{
			ShotInfo result = SaveImage(image, name);

			if (uploadCompleted != null)
				uploadCompleted(result);

			return result;
		}

		public XElement[] SaveSettings()
		{
			return new[]
			{
				new XElement("CurrentImageFormat", CurrentImageFormat),
				new XElement("FolderToSave", FolderToSave)
			};
		}

		public void LoadSettings(XElement[] settings)
		{
			if (settings == null || settings.Length == 0)
				return;

			XElement imageFormat = settings.FirstOrDefault(x => x.Name == "CurrentImageFormat");
			if (imageFormat != null)
				this.CurrentImageFormat = SupportedFormats.FirstOrDefault(x => x.ToString() == imageFormat.Value);

			XElement folderToSave = settings.FirstOrDefault(x => x.Name == "FolderToSave");
			if (folderToSave != null && !string.IsNullOrEmpty(folderToSave.Value))
				this.FolderToSave = folderToSave.Value;
		}
	}
}
