﻿using System;
using System.Collections.Generic;
using System.Reflection;
using CloudShot.Core.Types;
using CloudShot.Core.Utils;

namespace CloudShot.Utils
{
	static class AppHelpers
	{
		public static IEnumerable<Type> FindDerivedTypesFromAssembly(this Assembly assembly, Type baseType)
		{
			if (assembly == null)
				throw new ArgumentNullException("assembly", "Assembly must be defined");

			if (baseType == null)
				throw new ArgumentNullException("baseType", "Parent Type must be defined");

			Type[] types = assembly.GetExportedTypes();

			// works out the derived types
			foreach (var type in types)
			{
				// it must be a class
				if (!type.IsClass || type.IsAbstract)
					continue;

				if (baseType.IsInterface)
				{
					var it = type.GetInterface(baseType.FullName);

					if (it != null)
						// add it to result list
						yield return type;
				}
				else if (type.IsSubclassOf(baseType))
				{
					// add it to result list
					yield return type;
				}
			}
		}

		public static string GetFileName(ImageSaveFormat imageSaveFormat)
		{
			string date = string.Format("{0:yyMMdd_HHmmss}", DateTime.Now);
			return string.Format("shot_{0}.{1}", date, ImageTypeUtils.GetImageExtension(imageSaveFormat)).ToLower();
		}


	}
}
